@@ -226,7 +226,7 @@ bool Backend::reloadCache()
 
         if(!Ver.end()) {
             pkgCache::VerFileIterator VF = Ver.FileList();
-            QLatin1String origin(QLatin1String(VF.File().Origin()));
+            QString origin(QLatin1String(VF.File().Origin()));
             d->originMap[origin] = VF.File().Label();
             d->siteMap[origin] = VF.File().Site();
         }
@@ -487,7 +487,7 @@ PackageList Backend::upgradeablePackages() const
     PackageList upgradeablePackages;
 
     for (Package *package : d->packages) {
-        if (package->state() & Package::Upgradeable) {
+        if (package->staticState() & Package::Upgradeable) {
             upgradeablePackages << package;
         }
     }
@@ -716,13 +716,14 @@ CacheState Backend::currentCacheState() const
     int pkgSize = d->packages.size();
     state.reserve(pkgSize);
     for (int i = 0; i < pkgSize; ++i) {
-        state.append(d->packages[i]->state());
+        state.append(d->packages.at(i)->state());
     }
 
     return state;
 }
 
-QHash<Package::State, PackageList> Backend::stateChanges(CacheState oldState, PackageList excluded) const
+QHash<Package::State, PackageList> Backend::stateChanges(const CacheState &oldState,
+                                                         const PackageList &excluded) const
 {
     Q_D(const Backend);
 
@@ -776,10 +777,11 @@ void Backend::restoreCacheState(const CacheState &state)
     pkgDepCache *deps = d->cache->depCache();
     pkgDepCache::ActionGroup group(*deps);
 
-    for (int i = 0; i < d->packages.size(); ++i) {
-        Package *pkg = d->packages[i];
+    int packageCount = d->packages.size();
+    for (int i = 0; i < packageCount; ++i) {
+        Package *pkg = d->packages.at(i);
         int flags = pkg->state();
-        int oldflags = state[i];
+        int oldflags = state.at(i);
 
         if (oldflags == flags)
             continue;
@@ -923,7 +925,7 @@ void Backend::markPackages(const QApt::PackageList &packages, QApt::Package::Sta
         const pkgCache::PkgIterator &iter = package->packageIterator();
         switch (action) {
         case Package::ToInstall: {
-            int state = package->state();
+            int state = package->staticState();
             // Mark for install if not already installed, or if upgradeable
             if (!(state & Package::Installed) || (state & Package::Upgradeable)) {
                 package->setInstall();
@@ -941,7 +943,7 @@ void Backend::markPackages(const QApt::PackageList &packages, QApt::Package::Sta
             break;
         }
         case Package::ToReInstall: {
-            int state = package->state();
+            int state = package->staticState();
 
             if(state & Package::Installed
                && !(state & Package::NotDownloadable)
@@ -954,7 +956,7 @@ void Backend::markPackages(const QApt::PackageList &packages, QApt::Package::Sta
             package->setKeep();
             break;
         case Package::ToPurge: {
-            int state = package->state();
+            int state = package->staticState();
 
             if ((state & Package::Installed) || (state & Package::ResidualConfig)) {
                 package->setPurge();
