@@ -111,6 +111,8 @@ private:
                    RelationType rType, DependencyType dType);
 
     QSharedDataPointer<DependencyInfoPrivate> d;
+
+    friend class Package;
 };
 
 /**
@@ -123,4 +125,7 @@ typedef QList<DependencyInfo> DependencyItem;
 
 }
 
+Q_DECLARE_TYPEINFO(QApt::DependencyInfo, Q_MOVABLE_TYPE);
+Q_DECLARE_TYPEINFO(QList<QApt::DependencyInfo>, Q_MOVABLE_TYPE);
+
 #endif
