@@ -46,9 +46,11 @@
 
 #include <algorithm>
 
+// Own includes
 #include "backend.h"
 #include "cache.h"
 #include "config.h" // krazy:exclude=includes
+#include "markingerrorinfo.h"
 
 namespace QApt {
 
@@ -104,36 +106,39 @@ pkgCache::PkgFileIterator PackagePrivate::searchPkgFileIter(QLatin1String label,
         }
         ++verIter;
     }
-   found = pkgCache::PkgFileIterator(*packageIter.Cache());
-   return found;
+    found = pkgCache::PkgFileIterator(*packageIter.Cache());
+    return found;
 }
 
 QString PackagePrivate::getReleaseFileForOrigin(QLatin1String label, const QString &release) const
 {
-    pkgCache::PkgFileIterator found = searchPkgFileIter(label, release);
+    pkgCache::PkgFileIterator pkg = searchPkgFileIter(label, release);
 
-    if (found.end()) {
+    // Return empty if no package matches the given label and release
+    if (pkg.end())
         return QString();
-    }
 
-    // search for the matching meta-index
+    // Search for the matching meta-index
     pkgSourceList *list = backend->packageSourceList();
     pkgIndexFile *index;
 
-    if(list->FindIndex(found, index)) {
-        vector<metaIndex *>::const_iterator I;
-        for(I=list->begin(); I != list->end(); ++I) {
-            vector<pkgIndexFile *>  *ifv = (*I)->GetIndexFiles();
-            if(find(ifv->begin(), ifv->end(), index) != ifv->end()) {
-                QString uri = backend->config()->findDirectory("Dir::State::lists")
+    // Return empty if the source list doesn't contain an index for the package
+    if (!list->FindIndex(pkg, index))
+        return QString();
+
+    for (auto I = list->begin(); I != list->end(); ++I) {
+        vector<pkgIndexFile *> *ifv = (*I)->GetIndexFiles();
+        if (find(ifv->begin(), ifv->end(), index) == ifv->end())
+            continue;
+
+        // Construct release file path
+        QString uri = backend->config()->findDirectory("Dir::State::lists")
                 % QString::fromStdString(URItoFileName((*I)->GetURI()))
                 % QLatin1String("dists_")
                 % QString::fromStdString((*I)->GetDist())
                 % QLatin1String("_Release");
 
-                return uri;
-            }
-        }
+        return uri;
     }
 
     return QString();
@@ -144,21 +149,18 @@ void PackagePrivate::initStaticState(const pkgCache::VerIterator &ver, pkgDepCac
     int packageState = 0;
 
     if (!ver.end()) {
+        // Set flags exclusive to installed packages
         packageState |= QApt::Package::Installed;
 
         if (stateCache.CandidateVer && stateCache.Upgradable()) {
             packageState |= QApt::Package::Upgradeable;
-            if (stateCache.Keep()) {
+            if (stateCache.Keep())
                 packageState |= QApt::Package::Held;
-            }
         }
-
-        if (stateCache.Downgrade()) {
-            packageState |= QApt::Package::ToDowngrade;
-        }
-    } else {
+    } else
         packageState |= QApt::Package::NotInstalled;
-    }
+
+    // Broken/garbage statuses are constant until a cache reload
     if (stateCache.NowBroken()) {
         packageState |= QApt::Package::NowBroken;
     }
@@ -167,21 +169,6 @@ void PackagePrivate::initStaticState(const pkgCache::VerIterator &ver, pkgDepCac
         packageState |= QApt::Package::InstallBroken;
     }
 
-    if (packageIter->Flags & (pkgCache::Flag::Important |
-                             pkgCache::Flag::Essential)) {
-        packageState |= QApt::Package::IsImportant;
-    }
-
-    if (packageIter->CurrentState == pkgCache::State::ConfigFiles) {
-        packageState |= QApt::Package::ResidualConfig;
-    }
-
-    if (!stateCache.CandidateVer) {
-        packageState |= QApt::Package::NotDownloadable;
-    } else if (!stateCache.CandidateVerIter(*backend->cache()->depCache()).Downloadable()) {
-        packageState |= QApt::Package::NotDownloadable;
-    }
-
     if (stateCache.Garbage) {
         packageState |= QApt::Package::IsGarbage;
     }
@@ -194,6 +181,26 @@ void PackagePrivate::initStaticState(const pkgCache::VerIterator &ver, pkgDepCac
         packageState |= QApt::Package::InstallPolicyBroken;
     }
 
+    // Essential/important status can only be changed by cache reload
+    if (packageIter->Flags & (pkgCache::Flag::Important |
+                             pkgCache::Flag::Essential)) {
+        packageState |= QApt::Package::IsImportant;
+    }
+
+    if (packageIter->CurrentState == pkgCache::State::ConfigFiles) {
+        packageState |= QApt::Package::ResidualConfig;
+    }
+
+    // Packages will stay undownloadable until a sources file is refreshed
+    // and the cache is reloaded.
+    bool downloadable = true;
+    if (!stateCache.CandidateVer ||
+        stateCache.CandidateVerIter(*backend->cache()->depCache()).Downloadable())
+        downloadable = false;
+
+    if (!downloadable)
+        packageState |= QApt::Package::NotDownloadable;
+
     state |= packageState;
 
     staticStateCalculated = true;
@@ -232,13 +239,18 @@ QLatin1String Package::section() const
 QString Package::sourcePackage() const
 {
     QString sourcePackage;
+
+    // In the APT package record format, the only time when a "Source:" field
+    // is present is when the binary package name doesn't match the source
+    // name
     const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
     if (!ver.end()) {
         pkgRecords::Parser &rec = d->backend->records()->Lookup(ver.FileList());
-        sourcePackage = QLatin1String(rec.SourcePkg().c_str());
+        sourcePackage = QString::fromStdString(rec.SourcePkg());
     }
 
-    // If empty, use name. (Name would equal source package in that case)
+    // If the package record didn't have a "Source:" field, then this package's
+    // name must be the source package's name. (Or there isn't a record for this package)
     if (sourcePackage.isEmpty()) {
         sourcePackage = name();
     }
@@ -309,6 +321,8 @@ QString Package::maintainer() const
     if (!ver.end()) {
         pkgRecords::Parser &parser = d->backend->records()->Lookup(ver.FileList());
         maintainer = QString::fromUtf8(parser.Maintainer().data());
+        // This replacement prevents frontends from interpreting '<' as
+        // an HTML tag opening
         maintainer.replace(QLatin1Char('<'), QLatin1String("&lt;"));
     }
     return maintainer;
@@ -359,7 +373,8 @@ QString Package::upstreamVersion() const
 
 QString Package::upstreamVersion(const QString &version)
 {
-    return QString::fromStdString(_system->VS->UpstreamVersion(version.toStdString().c_str()));
+    QByteArray ver = version.toLatin1();
+    return QString::fromStdString(_system->VS->UpstreamVersion(ver.constData()));
 }
 
 QString Package::architecture() const
@@ -379,21 +394,20 @@ QStringList Package::availableVersions() const
     QStringList versions;
 
     // Get available Versions.
-    for (pkgCache::VerIterator Ver = d->packageIter.VersionList(); !Ver.end(); ++Ver) {
+    for (auto Ver = d->packageIter.VersionList(); !Ver.end(); ++Ver) {
 
         // We always take the first available version.
         pkgCache::VerFileIterator VF = Ver.FileList();
-        if (!VF.end()) {
-            pkgCache::PkgFileIterator File = VF.File();
+        if (VF.end())
+            continue;
 
-            if (File->Archive != 0) {
-                versions.append(QLatin1String(Ver.VerStr()) % QLatin1Literal(" (") %
-                QLatin1String(File.Archive()) % QLatin1Char(')'));
-            } else {
-                versions.append(QLatin1String(Ver.VerStr()) % QLatin1Literal(" (") %
-                QLatin1String(File.Site()) % QLatin1Char(')'));
-            }
-        }
+        pkgCache::PkgFileIterator File = VF.File();
+
+        // Files without an archive will have a site
+        QString archive = (File->Archive) ? QLatin1String(File.Archive()) :
+                                            QLatin1String(File.Site());
+        versions.append(QLatin1String(Ver.VerStr()) % QLatin1String(" (") %
+                        archive % ')');
     }
 
     return versions;
@@ -421,11 +435,10 @@ QString Package::availableVersion() const
 QString Package::priority() const
 {
     const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
-    if (!ver.end()) {
-        return QLatin1String(ver.PriorityType());
-    }
+    if (ver.end())
+        return QString();
 
-    return QString();
+    return QLatin1String(ver.PriorityType());
 }
 
 QStringList Package::installedFilesList() const
@@ -475,27 +488,25 @@ QString Package::origin() const
 {
     const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
-    if(!Ver.end()) {
-         pkgCache::VerFileIterator VF = Ver.FileList();
-         return QLatin1String(VF.File().Origin());
-    }
+    if(Ver.end())
+        return QString();
 
-    return QString();
+    pkgCache::VerFileIterator VF = Ver.FileList();
+    return QLatin1String(VF.File().Origin());
 }
 
 QStringList Package::archives() const
 {
     const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
-    if(!Ver.end()) {
-        QStringList archiveList;
-        for (pkgCache::VerFileIterator VF = Ver.FileList(); !VF.end(); ++VF) {
-            archiveList << QLatin1String(VF.File().Archive());
-        }
-            return archiveList;
-    }
+    if(Ver.end())
+        return QStringList();
 
-    return QStringList();
+    QStringList archiveList;
+    for (auto VF = Ver.FileList(); !VF.end(); ++VF)
+        archiveList << QLatin1String(VF.File().Archive());
+
+    return archiveList;
 }
 
 QString Package::component() const
@@ -514,60 +525,51 @@ QString Package::component() const
 
 QByteArray Package::md5Sum() const
 {
-    QByteArray md5Sum;
-
     const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
 
-    if(ver.end()) {
-        return md5Sum;
-    }
+    if(ver.end())
+        return QByteArray();
 
     pkgRecords::Parser &rec = d->backend->records()->Lookup(ver.FileList());
-    md5Sum = rec.MD5Hash().c_str();
 
-    return md5Sum;
+    return rec.MD5Hash().c_str();
 
 }
 
 QUrl Package::changelogUrl() const
 {
-    QUrl url;
-
     const pkgCache::VerIterator &ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
-    if (ver.end()) {
-        return url;
-    }
+    if (ver.end())
+        return QUrl();
 
     pkgRecords::Parser &rec = d->backend->records()->Lookup(ver.FileList());
 
-    QString path = QLatin1String(rec.FileName().c_str());
-    path = path.left(path.lastIndexOf(QLatin1Char('/')) + 1);
-
+    // Find the latest version for the latest changelog
     QString versionString;
-    if (!availableVersion().isEmpty()) {
+    if (!availableVersion().isEmpty())
         versionString = availableVersion();
-    }
 
+    // Epochs in versions are ignored on changelog servers
     if (versionString.contains(QLatin1Char(':'))) {
         QStringList epochVersion = versionString.split(QLatin1Char(':'));
         // If the version has an epoch, take the stuff after the epoch
-        versionString = epochVersion[1];
+        versionString = epochVersion.at(1);
     }
 
-    path += sourcePackage() % QLatin1Char('_') % versionString % QLatin1Char('/');
-
+    // Create URL in form using the correct server, file path, and file suffix
     Config *config = d->backend->config();
     QString server = config->readEntry(QLatin1String("Apt::Changelogs::Server"),
                                        QLatin1String("http://packages.debian.org/changelogs"));
 
-    if(!server.contains(QLatin1String("debian"))) {
-        url = QUrl(server % QLatin1Char('/') % path % QLatin1Literal("changelog"));
-    } else {
-        // Debian servers use changelog.txt
-        url = QUrl(server % QLatin1Char('/') % path % QLatin1Literal("changelog.txt"));
-    }
+    QString path = QLatin1String(rec.FileName().c_str());
+    path = path.left(path.lastIndexOf(QLatin1Char('/')) + 1);
+    path += sourcePackage() % '_' % versionString % '/';
 
-    return url;
+    bool fromDebian = server.contains(QLatin1String("debian"));
+    QString suffix = fromDebian ? QLatin1String("changelog.txt")
+                                : QLatin1String("changelog");
+
+    return QUrl(server % '/' % path % suffix);
 }
 
 QUrl Package::screenshotUrl(QApt::ScreenshotType type) const
@@ -589,16 +591,16 @@ QUrl Package::screenshotUrl(QApt::ScreenshotType type) const
     return url;
 }
 
-QString Package::supportedUntil() const
+QDateTime Package::supportedUntil() const
 {
     if (!isSupported()) {
-        return QString();
+        return QDateTime();
     }
 
     QFile lsb_release(QLatin1String("/etc/lsb-release"));
     if (!lsb_release.open(QFile::ReadOnly)) {
         // Though really, your system is screwed if this happens...
-        return QString();
+        return QDateTime();
     }
 
     pkgTagSection sec;
@@ -625,7 +627,7 @@ QString Package::supportedUntil() const
 
     if(!FileExists(releaseFile.toStdString())) {
         // happens e.g. when there is no release file and is harmless
-        return QString();
+        return QDateTime();
     }
 
     // read the relase file
@@ -634,7 +636,7 @@ QString Package::supportedUntil() const
     tag.Step(sec);
 
     if(!RFC1123StrToTime(sec.FindS("Date").data(), releaseDate)) {
-        return QString();
+        return QDateTime();
     }
 
     // Default to 18m in case the package has no "supported" field
@@ -657,7 +659,7 @@ QString Package::supportedUntil() const
         supportEnd = QDateTime::fromTime_t(releaseDate).addYears(supportTime);
     }
 
-    return supportEnd.toString(QLatin1String("MMMM yyyy"));
+    return supportEnd;
 }
 
 QString Package::controlField(QLatin1String name) const
@@ -746,6 +748,17 @@ int Package::state() const
    return packageState | d->state;
 }
 
+int Package::staticState() const
+{
+    if (!d->staticStateCalculated) {
+        const pkgCache::VerIterator &ver = d->packageIter.CurrentVer();
+        pkgDepCache::StateCache &stateCache = (*d->backend->cache()->depCache())[d->packageIter];
+        d->initStaticState(ver, stateCache);
+    }
+
+    return d->state;
+}
+
 int Package::compareVersion(const QString &v1, const QString &v2)
 {
     // Make deep copies of toStdString(), since otherwise they would
@@ -1054,24 +1067,17 @@ QStringList Package::enhancedByList() const
 }
 
 
-QHash<int, QHash<QString, QVariantMap> > Package::brokenReason() const
+QList<QApt::MarkingErrorInfo> Package::brokenReason() const
 {
     const pkgCache::VerIterator &Ver = (*d->backend->cache()->depCache()).GetCandidateVer(d->packageIter);
-
-    QHash<QString, QVariantMap> wrongCandidate;
-    QHash<QString, QVariantMap> depNotInstallable;
-    QHash<QString, QVariantMap> virtualPackage;
-
-    // failTrain represents brokenness, but also the complexity of this
-    // function...
-    QHash<int, QHash<QString, QVariantMap> > failTrain;
+    QList<MarkingErrorInfo> reasons;
 
     // check if there is actually something to install
     if (!Ver) {
-        QHash<QString, QVariantMap> parentNotInstallable;
-        parentNotInstallable[name()] = QVariantMap();
-        failTrain[QApt::ParentNotInstallable] = parentNotInstallable;
-        return failTrain;
+        QApt::DependencyInfo info(name(), QString(), NoOperand, InvalidType);
+        QApt::MarkingErrorInfo error(QApt::ParentNotInstallable, info);
+        reasons.append(error);
+        return reasons;
     }
 
     for (pkgCache::DepIterator D = Ver.DependsList(); !D.end();) {
@@ -1104,58 +1110,36 @@ QHash<int, QHash<QString, QVariantMap> > Package::brokenReason() const
                 // Happens when a package needs an upgraded dep, but the dep won't
                 // upgrade. Example:
                 // "apt 0.5.4 but 0.5.3 is to be installed"
-                QVariantMap failReason;
-                failReason[QLatin1String("Relation")] = QLatin1String(End.DepType());
-                failReason[QLatin1String("RequiredVersion")] = requiredVersion;
-                failReason[QLatin1String("CandidateVersion")] = QLatin1String(Ver.VerStr());
-                if (Start != End) {
-                    failReason[QLatin1String("IsFirstOr")] = true;
-                }
-
                 QString targetName = QLatin1String(Start.TargetPkg().Name());
-                wrongCandidate[targetName] = failReason;
+                QApt::DependencyType relation = (QApt::DependencyType)End->Type;
+
+                QApt::DependencyInfo errorInfo(targetName, requiredVersion,
+                                               NoOperand, relation);
+                QApt::MarkingErrorInfo error(QApt::WrongCandidateVersion, errorInfo);
+                reasons.append(error);
             } else { // We have the package, but for some reason it won't be installed
                 // In this case, the required version does not exist at all
-                if ((*d->backend->cache()->depCache())[Targ].CandidateVerIter(*d->backend->cache()->depCache()).end()) {
-                    QVariantMap failReason;
-                    failReason[QLatin1String("Relation")] = QLatin1String(End.DepType());
-                    failReason[QLatin1String("RequiredVersion")] = requiredVersion;
-                    if (Start != End) {
-                        failReason[QLatin1String("IsFirstOr")] = true;
-                    }
+                QString targetName = QLatin1String(Start.TargetPkg().Name());
+                QApt::DependencyType relation = (QApt::DependencyType)End->Type;
 
-                    QString targetName = QLatin1String(Start.TargetPkg().Name());
-                    depNotInstallable[targetName] = failReason;
-                } else {
-                    // Who knows why it won't be installed? Getting here means we have no good reason
-                    QVariantMap failReason;
-                    failReason[QLatin1String("Relation")] = QLatin1String(End.DepType());
-                    if (Start != End) {
-                        failReason[QLatin1String("IsFirstOr")] = true;
-                    }
-
-                    QString targetName = QLatin1String(Start.TargetPkg().Name());
-                    depNotInstallable[targetName] = failReason;
-                }
+                QApt::DependencyInfo errorInfo(targetName, requiredVersion,
+                                               NoOperand, relation);
+                QApt::MarkingErrorInfo error(QApt::DepNotInstallable, errorInfo);
+                reasons.append(error);
             }
         } else {
             // Ok, candidate has provides. We're a virtual package
-            QVariantMap failReason;
-            failReason[QLatin1String("Relation")] = QLatin1String(End.DepType());
-            if (Start != End) {
-                failReason[QLatin1String("IsFirstOr")] = true;
-            }
-
             QString targetName = QLatin1String(Start.TargetPkg().Name());
-            virtualPackage[targetName] = failReason;
+            QApt::DependencyType relation = (QApt::DependencyType)End->Type;
+
+            QApt::DependencyInfo errorInfo(targetName, QString(),
+                                           NoOperand, relation);
+            QApt::MarkingErrorInfo error(QApt::VirtualPackage, errorInfo);
+            reasons.append(error);
         }
     }
 
-    failTrain[QApt::WrongCandidateVersion] = wrongCandidate;
-    failTrain[QApt::DepNotInstallable] = depNotInstallable;
-    failTrain[QApt::VirtualPackage] = virtualPackage;
-
-    return failTrain;
+    return reasons;
 }
 
 bool Package::isTrusted() const
@@ -1297,40 +1281,30 @@ void Package::setPurge()
 
 bool Package::setVersion(const QString &version)
 {
-    QLatin1String defaultCandVer("");
     pkgDepCache::StateCache &state = (*d->backend->cache()->depCache())[d->packageIter];
-    if (state.CandVersion != nullptr) {
-        defaultCandVer = QLatin1String(state.CandVersion);
-    }
+    QLatin1String defaultCandVer(state.CandVersion);
 
     bool isDefault = (version == defaultCandVer);
     pkgVersionMatch Match(version.toLatin1().constData(), pkgVersionMatch::Version);
     const pkgCache::VerIterator &Ver = Match.Find(d->packageIter);
 
-    if (Ver.end()) {
+    if (Ver.end())
         return false;
-    }
 
     d->backend->cache()->depCache()->SetCandidateVersion(Ver);
 
-    string archive;
-    for (pkgCache::VerFileIterator VF = Ver.FileList();
-         VF.end() == false;
-         ++VF)
-    {
+    for (auto VF = Ver.FileList(); !VF.end(); ++VF) {
         if (!VF.File() || !VF.File().Archive())
             continue;
 
-        archive = VF.File().Archive();
-        d->backend->cache()->depCache()->SetCandidateRelease(Ver, archive);
+        d->backend->cache()->depCache()->SetCandidateRelease(Ver, VF.File().Archive());
         break;
     }
 
-    if (isDefault) {
+    if (isDefault)
         d->state &= ~OverrideVersion;
-    } else {
+    else
         d->state |= OverrideVersion;
-    }
 
     return true;
 }
